/**
 *  Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the license found in the
 *  LICENSE-examples file in the root directory of this source tree.
 */

'use strict';

exports.__esModule = true;

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

/**
 * ExecuteButton
 *
 * What a nice round shiny button. Shows a drop-down when there are multiple
 * queries to run.
 */

var ExecuteButton = (function (_React$Component) {
  _inherits(ExecuteButton, _React$Component);

  _createClass(ExecuteButton, null, [{
    key: 'propTypes',
    value: {
      onClick: _react.PropTypes.func,
      isRunning: _react.PropTypes.bool,
      operations: _react.PropTypes.array
    },
    enumerable: true
  }]);

  function ExecuteButton(props) {
    var _this = this;

    _classCallCheck(this, ExecuteButton);

    _React$Component.call(this, props);

    this._onClick = function () {
      if (_this.props.isRunning) {
        _this.props.onStop();
      } else {
        _this.props.onRun();
      }
    };

    this._onOptionSelected = function (operation) {
      _this.setState({ optionsOpen: false });
      _this.props.onRun(operation.name && operation.name.value);
    };

    this._onOptionsOpen = function (downEvent) {
      var initialPress = true;
      var downTarget = downEvent.target;
      _this.setState({ highlight: null, optionsOpen: true });

      var _onMouseUp = function onMouseUp(upEvent) {
        if (initialPress && upEvent.target === downTarget) {
          initialPress = false;
        } else {
          document.removeEventListener('mouseup', _onMouseUp);
          _onMouseUp = null;
          _this.setState({ optionsOpen: false });
        }
      };

      document.addEventListener('mouseup', _onMouseUp);
    };

    this.state = {
      optionsOpen: false,
      highlight: null
    };
  }

  ExecuteButton.prototype.render = function render() {
    var _this2 = this;

    var operations = this.props.operations;
    var optionsOpen = this.state.optionsOpen;
    var hasOptions = operations && operations.length > 1;

    var options = null;
    if (hasOptions && optionsOpen) {
      (function () {
        var highlight = _this2.state.highlight;
        options = _react2['default'].createElement(
          'ul',
          { className: 'execute-options' },
          operations.map(function (operation) {
            return _react2['default'].createElement(
              'li',
              {
                className: operation === highlight && 'selected',
                onMouseOver: function () {
                  return _this2.setState({ highlight: operation });
                },
                onMouseOut: function () {
                  return _this2.setState({ highlight: null });
                },
                onMouseUp: function () {
                  return _this2._onOptionSelected(operation);
                } },
              operation.name ? operation.name.value : '<Unnamed>'
            );
          })
        );
      })();
    }

    // Allow click event if there is a running query or if there are not options
    // for which operation to run.
    var onClick = undefined;
    if (this.props.isRunning || !hasOptions) {
      onClick = this._onClick;
    }

    // Allow mouse down if there is no running query, there are options for
    // which operation to run, and the dropdown is currently closed.
    var onMouseDown = undefined;
    if (!this.props.isRunning && hasOptions && !optionsOpen) {
      onMouseDown = this._onOptionsOpen;
    }

    return _react2['default'].createElement(
      'div',
      { className: 'execute-button-wrap' },
      _react2['default'].createElement(
        'button',
        {
          className: 'execute-button',
          onMouseDown: onMouseDown,
          onClick: onClick,
          title: 'Execute Query (Ctrl-Enter)' },
        _react2['default'].createElement(
          'svg',
          { width: '34', height: '34' },
          this.props.isRunning ? _react2['default'].createElement('path', { d: 'M 10 10 L 23 10 L 23 23 L 10 23 z' }) : _react2['default'].createElement('path', { d: 'M 11 9 L 24 16 L 11 23 z' })
        )
      ),
      options
    );
  };

  return ExecuteButton;
})(_react2['default'].Component);

exports.ExecuteButton = ExecuteButton;