/**
 *  Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the license found in the
 *  LICENSE-examples file in the root directory of this source tree.
 */

'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _graphql = require('graphql');

var _ExecuteButton = require('./ExecuteButton');

var _ToolbarButton = require('./ToolbarButton');

var _QueryEditor = require('./QueryEditor');

var _VariableEditor = require('./VariableEditor');

var _ResultViewer = require('./ResultViewer');

var _DocExplorer = require('./DocExplorer');

var _utilityGetQueryFacts = require('../utility/getQueryFacts');

var _utilityGetQueryFacts2 = _interopRequireDefault(_utilityGetQueryFacts);

var _utilityDebounce = require('../utility/debounce');

var _utilityDebounce2 = _interopRequireDefault(_utilityDebounce);

var _utilityFind = require('../utility/find');

var _utilityFind2 = _interopRequireDefault(_utilityFind);

var _utilityFillLeafs = require('../utility/fillLeafs');

var _utilityElementPosition = require('../utility/elementPosition');

var _utilityIntrospectionQueries = require('../utility/introspectionQueries');

/**
 * GraphiQL
 *
 * This React component is responsible for rendering the GraphiQL editor.
 *
 * Props:
 *
 *   - fetcher: a function which accepts GraphQL-HTTP parameters and returns
 *     a Promise or Observable which resolves to the GraphQL parsed
 *     JSON response.
 *
 *   - schema: a GraphQLSchema instance or `null` if one is not to be used.
 *     If `undefined` is provided, GraphiQL will send an introspection query
 *     using the fetcher to produce a schema.
 *
 *   - query: an optional GraphQL string to use as the initial displayed query,
 *     if `undefined` is provided, the stored query or defaultQuery will
 *     be used.
 *
 *   - variables: an optional GraphQL string to use as the initial displayed
 *     query variables, if `undefined` is provided, the stored variables will
 *     be used.
 *
 *   - response: an optional JSON string to use as the initial displayed
 *     response. If not provided, no response will be initialy shown. You might
 *     provide this if illustrating the result of the initial query.
 *
 *   - storage: an instance of [Storage][] GraphiQL will use to persist state.
 *     Only `getItem` and `setItem` are called. Default: window.localStorage
 *
 *   - defaultQuery: an optional GraphQL string to use when no query is provided
 *     and no stored query exists from a previous session. If `undefined` is
 *     provided, GraphiQL will use its own default query.
 *
 *   - onEditQuery: an optional function which will be called when the Query
 *     editor changes. The argument to the function will be the query string.
 *
 *   - onEditVariables: an optional function which will be called when the Query
 *     varible editor changes. The argument to the function will be the
 *     variables string.
 *
 *   - getDefaultFieldNames: an optional function used to provide default fields
 *     to non-leaf fields which invalidly lack a selection set.
 *     Accepts a GraphQLType instance and returns an array of field names.
 *     If not provided, a default behavior will be used.
 *
 * Children:
 *
 *   - <GraphiQL.Logo> Replace the GraphiQL logo with your own.
 *
 *   - <GraphiQL.Toolbar> Add a custom toolbar above GraphiQL.
 *
 *   - <GraphiQL.ToolbarButton> Add a button to the toolbar above GraphiQL.
 *
 *   - <GraphiQL.Footer> Add a custom footer below GraphiQL Results.
 *
 *
 * [Storage]: https://developer.mozilla.org/en-US/docs/Web/API/Storage
 */

var GraphiQL = (function (_React$Component) {
  _inherits(GraphiQL, _React$Component);

  /**
   * Inspect the query, automatically filling in selection sets for non-leaf
   * fields which do not yet have them.
   *
   * @public
   */

  GraphiQL.prototype.autoCompleteLeafs = function autoCompleteLeafs() {
    var _fillLeafs = _utilityFillLeafs.fillLeafs(this.state.schema, this.state.query, this.props.getDefaultFieldNames);

    var insertions = _fillLeafs.insertions;
    var result = _fillLeafs.result;

    if (insertions && insertions.length > 0) {
      var editor = this.refs.queryEditor.getCodeMirror();
      editor.operation(function () {
        var cursor = editor.getCursor();
        var cursorIndex = editor.indexFromPos(cursor);
        editor.setValue(result);
        var added = 0;
        var markers = insertions.map(function (_ref) {
          var index = _ref.index;
          var string = _ref.string;
          return editor.markText(editor.posFromIndex(index + added), editor.posFromIndex(index + (added += string.length)), {
            className: 'autoInsertedLeaf',
            clearOnEnter: true,
            title: 'Automatically added leaf fields'
          });
        });
        setTimeout(function () {
          return markers.forEach(function (marker) {
            return marker.clear();
          });
        }, 7000);
        var newCursorIndex = cursorIndex;
        insertions.forEach(function (_ref2) {
          var index = _ref2.index;
          var string = _ref2.string;

          if (index < cursorIndex) {
            newCursorIndex += string.length;
          }
        });
        var newCursor = editor.posFromIndex(newCursorIndex);
        editor.setCursor(newCursor);
      });
    }

    return result;
  };

  // Lifecycle

  _createClass(GraphiQL, null, [{
    key: 'propTypes',
    value: {
      fetcher: _react.PropTypes.func.isRequired,
      schema: _react.PropTypes.instanceOf(_graphql.GraphQLSchema),
      query: _react.PropTypes.string,
      response: _react.PropTypes.string,
      storage: _react.PropTypes.shape({
        getItem: _react.PropTypes.func,
        setItem: _react.PropTypes.func
      }),
      defaultQuery: _react.PropTypes.string,
      variables: _react.PropTypes.string,
      onEditQuery: _react.PropTypes.func,
      onEditVariables: _react.PropTypes.func,
      getDefaultFieldNames: _react.PropTypes.func
    },
    enumerable: true
  }]);

  function GraphiQL(props) {
    var _this = this;

    _classCallCheck(this, GraphiQL);

    _React$Component.call(this, props);

    // Ensure props are correct

    this._runQuery = function (selectedOperation) {
      _this._editorQueryID++;
      var queryID = _this._editorQueryID;

      // Use the edited query after autoCompleteLeafs() runs or,
      // in case autoCompletion fails (the function returns undefined),
      // the current query from the editor.
      var editedQuery = _this.autoCompleteLeafs() || _this.state.query;
      var variables = _this.state.variables;
      var operationName = selectedOperation || _this.state.selectedOperation;

      // _fetchQuery may return a subscription.
      var subscription = _this._fetchQuery(editedQuery, variables, operationName, function (result) {
        if (queryID === _this._editorQueryID) {
          _this.setState({
            isWaitingForResponse: false,
            response: JSON.stringify(result, null, 2)
          });
        }
      });

      _this.setState({
        isWaitingForResponse: true,
        response: null,
        subscription: subscription,
        selectedOperation: operationName
      });
    };

    this._stopQuery = function () {
      var subscription = _this.state.subscription;
      _this.setState({
        isWaitingForResponse: false,
        subscription: null
      });
      if (subscription) {
        subscription.unsubscribe();
      }
      return;
    };

    this._keyHandler = function (event) {
      // "Run Query" event.
      if ((event.metaKey || event.ctrlKey) && event.keyCode === 13) {
        event.preventDefault();
        _this._runQueryAtCursor();
      }
    };

    this._prettifyQuery = function () {
      var query = _graphql.print(_graphql.parse(_this.state.query));
      var editor = _this.refs.queryEditor.getCodeMirror();
      editor.setValue(query);
    };

    this._onEditQuery = function (value) {
      if (_this.state.schema) {
        _this._updateQueryFacts(value);
      }
      _this.setState({ query: value });
      if (_this.props.onEditQuery) {
        return _this.props.onEditQuery(value);
      }
    };

    this._updateQueryFacts = _utilityDebounce2['default'](500, function (value) {
      var queryFacts = _utilityGetQueryFacts2['default'](_this.state, _this.state.schema, value);
      if (queryFacts) {
        _this.setState(queryFacts);
      }
    });

    this._onEditVariables = function (value) {
      _this.setState({ variables: value });
      if (_this.props.onEditVariables) {
        _this.props.onEditVariables(value);
      }
    };

    this._onHintInformationRender = function (elem) {
      elem.addEventListener('click', _this._onClickHintInformation);

      var onRemoveFn;
      elem.addEventListener('DOMNodeRemoved', onRemoveFn = function () {
        elem.removeEventListener('DOMNodeRemoved', onRemoveFn);
        elem.removeEventListener('click', _this._onClickHintInformation);
      });
    };

    this._onClickHintInformation = function (event) {
      if (event.target.className === 'typeName') {
        var typeName = event.target.innerHTML;
        var schema = _this.state.schema;
        if (schema) {
          var type = schema.getType(typeName);
          if (type) {
            _this.setState({ docExplorerOpen: true }, function () {
              _this.refs.docExplorer.showDoc(type);
            });
          }
        }
      }
    };

    this._onToggleDocs = function () {
      _this.setState({ docExplorerOpen: !_this.state.docExplorerOpen });
    };

    this._onResizeStart = function (downEvent) {
      if (!_this._didClickDragBar(downEvent)) {
        return;
      }

      downEvent.preventDefault();

      var offset = downEvent.clientX - _utilityElementPosition.getLeft(downEvent.target);

      var onMouseMove = function onMouseMove(moveEvent) {
        if (moveEvent.buttons === 0) {
          return onMouseUp();
        }

        var editorBar = _reactDom2['default'].findDOMNode(_this.refs.editorBar);
        var leftSize = moveEvent.clientX - _utilityElementPosition.getLeft(editorBar) - offset;
        var rightSize = editorBar.clientWidth - leftSize;
        _this.setState({ editorFlex: leftSize / rightSize });
      };

      var onMouseUp = (function (_onMouseUp) {
        function onMouseUp() {
          return _onMouseUp.apply(this, arguments);
        }

        onMouseUp.toString = function () {
          return _onMouseUp.toString();
        };

        return onMouseUp;
      })(function () {
        document.removeEventListener('mousemove', onMouseMove);
        document.removeEventListener('mouseup', onMouseUp);
        onMouseMove = null;
        onMouseUp = null;
      });

      document.addEventListener('mousemove', onMouseMove);
      document.addEventListener('mouseup', onMouseUp);
    };

    this._onDocsResizeStart = function (downEvent) {
      downEvent.preventDefault();

      var hadWidth = _this.state.docExplorerWidth;
      var offset = downEvent.clientX - _utilityElementPosition.getLeft(downEvent.target);

      var onMouseMove = function onMouseMove(moveEvent) {
        if (moveEvent.buttons === 0) {
          return onMouseUp();
        }

        var app = _reactDom2['default'].findDOMNode(_this);
        var cursorPos = moveEvent.clientX - _utilityElementPosition.getLeft(app) - offset;
        var docsSize = app.clientWidth - cursorPos;

        if (docsSize < 100) {
          _this.setState({ docExplorerOpen: false });
        } else {
          _this.setState({
            docExplorerOpen: true,
            docExplorerWidth: Math.min(docsSize, 650)
          });
        }
      };

      var onMouseUp = (function (_onMouseUp2) {
        function onMouseUp() {
          return _onMouseUp2.apply(this, arguments);
        }

        onMouseUp.toString = function () {
          return _onMouseUp2.toString();
        };

        return onMouseUp;
      })(function () {
        if (!_this.state.docExplorerOpen) {
          _this.setState({ docExplorerWidth: hadWidth });
        }

        document.removeEventListener('mousemove', onMouseMove);
        document.removeEventListener('mouseup', onMouseUp);
        onMouseMove = null;
        onMouseUp = null;
      });

      document.addEventListener('mousemove', onMouseMove);
      document.addEventListener('mouseup', onMouseUp);
    };

    this._onVariableResizeStart = function (downEvent) {
      downEvent.preventDefault();

      var didMove = false;
      var wasOpen = _this.state.variableEditorOpen;
      var hadHeight = _this.state.variableEditorHeight;
      var offset = downEvent.clientY - _utilityElementPosition.getTop(downEvent.target);

      var onMouseMove = function onMouseMove(moveEvent) {
        if (moveEvent.buttons === 0) {
          return onMouseUp();
        }

        didMove = true;

        var editorBar = _reactDom2['default'].findDOMNode(_this.refs.editorBar);
        var topSize = moveEvent.clientY - _utilityElementPosition.getTop(editorBar) - offset;
        var bottomSize = editorBar.clientHeight - topSize;
        if (bottomSize < 60) {
          _this.setState({
            variableEditorOpen: false,
            variableEditorHeight: hadHeight
          });
        } else {
          _this.setState({
            variableEditorOpen: true,
            variableEditorHeight: bottomSize
          });
        }
      };

      var onMouseUp = (function (_onMouseUp3) {
        function onMouseUp() {
          return _onMouseUp3.apply(this, arguments);
        }

        onMouseUp.toString = function () {
          return _onMouseUp3.toString();
        };

        return onMouseUp;
      })(function () {
        if (!didMove) {
          _this.setState({ variableEditorOpen: !wasOpen });
        }

        document.removeEventListener('mousemove', onMouseMove);
        document.removeEventListener('mouseup', onMouseUp);
        onMouseMove = null;
        onMouseUp = null;
      });

      document.addEventListener('mousemove', onMouseMove);
      document.addEventListener('mouseup', onMouseUp);
    };

    if (typeof props.fetcher !== 'function') {
      throw new TypeError('GraphiQL requires a fetcher function.');
    }

    // Cache the storage instance
    this._storage = props.storage || window.localStorage;

    // Determine the initial query to display.
    var query = props.query !== undefined ? props.query : this._storageGet('query') !== undefined ? this._storageGet('query') : props.defaultQuery !== undefined ? props.defaultQuery : defaultQuery;

    // Determine the initial variables to display.
    var variables = props.variables !== undefined ? props.variables : this._storageGet('variables');

    // Get the initial query facts.
    var queryFacts = _utilityGetQueryFacts2['default']({}, props.schema, query);

    // Initialize state
    this.state = _extends({
      schema: props.schema,
      query: query,
      variables: variables,
      response: props.response,
      editorFlex: this._storageGet('editorFlex') || 1,
      variableEditorOpen: Boolean(variables),
      variableEditorHeight: this._storageGet('variableEditorHeight') || 200,
      docExplorerOpen: false,
      docExplorerWidth: this._storageGet('docExplorerWidth') || 350,
      isWaitingForResponse: false,
      subscription: null
    }, queryFacts);

    // Ensure only the last executed editor query is rendered.
    this._editorQueryID = 0;

    // Subscribe to the browser window closing, treating it as an unmount.
    if (typeof window === 'object') {
      window.addEventListener('beforeunload', function () {
        return _this.componentWillUnmount();
      });
    }
  }

  // Configure the UI by providing this Component as a child of GraphiQL.

  // When the component is about to unmount, store any persistable state, such
  // that when the component is remounted, it will use the last used values.

  GraphiQL.prototype.componentWillUnmount = function componentWillUnmount() {
    this._storageSet('query', this.state.query);
    this._storageSet('variables', this.state.variables);
    this._storageSet('editorFlex', this.state.editorFlex);
    this._storageSet('variableEditorHeight', this.state.variableEditorHeight);
    this._storageSet('docExplorerWidth', this.state.docExplorerWidth);

    document.removeEventListener('keydown', this._keyHandler, true);
  };

  GraphiQL.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var nextSchema = this.state.schema;
    var nextQuery = this.state.query;
    var nextVariables = this.state.variables;
    var nextResponse = this.state.response;
    var queryFacts = undefined;
    if (nextProps.schema !== undefined) {
      nextSchema = nextProps.schema;
    }
    if (nextProps.query !== undefined) {
      nextQuery = nextProps.query;
    }
    if (nextProps.variables !== undefined) {
      nextVariables = nextProps.variables;
    }
    if (nextProps.response !== undefined) {
      nextResponse = nextProps.response;
    }
    if (nextSchema && nextQuery && (nextSchema !== this.state.schema || nextQuery !== this.state.query)) {
      queryFacts = _utilityGetQueryFacts2['default'](this.state, nextSchema, nextQuery);
    }
    this.setState(_extends({
      schema: nextSchema,
      query: nextQuery,
      variables: nextVariables,
      response: nextResponse
    }, queryFacts));
  };

  GraphiQL.prototype.componentDidMount = function componentDidMount() {
    // Ensure a form of a schema exists (including `null`) and
    // if not, fetch one using an introspection query.
    this._ensureOfSchema();

    // Add shortcut for running a query.
    document.addEventListener('keydown', this._keyHandler, true);
  };

  GraphiQL.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    // When UI-altering state changes, simulate a window resize event so all
    // CodeMirror instances become properly rendered.
    if (this.state.variableEditorOpen !== prevState.variableEditorOpen || this.state.variableEditorHeight !== prevState.variableEditorHeight) {
      window.dispatchEvent(new Event('resize'));
    }
  };

  GraphiQL.prototype.render = function render() {
    var children = [];
    _react2['default'].Children.forEach(this.props.children, function (child) {
      children.push(child);
    });

    var logo = _utilityFind2['default'](children, function (child) {
      return child.type === GraphiQL.Logo;
    }) || _react2['default'].createElement(GraphiQL.Logo, null);

    var toolbar = _utilityFind2['default'](children, function (child) {
      return child.type === GraphiQL.Toolbar;
    }) || _react2['default'].createElement(GraphiQL.Toolbar, null);

    var footer = _utilityFind2['default'](children, function (child) {
      return child.type === GraphiQL.Footer;
    });

    var queryWrapStyle = {
      WebkitFlex: this.state.editorFlex,
      flex: this.state.editorFlex
    };

    var docWrapStyle = {
      display: this.state.docExplorerOpen ? 'block' : 'none',
      width: this.state.docExplorerWidth
    };

    var variableOpen = this.state.variableEditorOpen;
    var variableStyle = {
      height: variableOpen ? this.state.variableEditorHeight : null
    };

    return _react2['default'].createElement(
      'div',
      { id: 'graphiql-container' },
      _react2['default'].createElement(
        'div',
        { className: 'editorWrap' },
        _react2['default'].createElement(
          'div',
          { className: 'topBarWrap' },
          _react2['default'].createElement(
            'div',
            { className: 'topBar' },
            logo,
            _react2['default'].createElement(_ExecuteButton.ExecuteButton, {
              isRunning: Boolean(this.state.subscription),
              onRun: this._runQuery,
              onStop: this._stopQuery,
              operations: this.state.operations
            }),
            _react2['default'].createElement(GraphiQL.ToolbarButton, {
              onClick: this._prettifyQuery,
              title: 'Prettify Query',
              label: 'Prettify'
            }),
            toolbar
          ),
          !this.state.docExplorerOpen && _react2['default'].createElement(
            'button',
            { className: 'docExplorerShow', onClick: this._onToggleDocs },
            'Docs'
          )
        ),
        _react2['default'].createElement(
          'div',
          {
            ref: 'editorBar',
            className: 'editorBar',
            onMouseDown: this._onResizeStart
          },
          _react2['default'].createElement(
            'div',
            { className: 'queryWrap', style: queryWrapStyle },
            _react2['default'].createElement(_QueryEditor.QueryEditor, {
              ref: 'queryEditor',
              schema: this.state.schema,
              value: this.state.query,
              onEdit: this._onEditQuery,
              onHintInformationRender: this._onHintInformationRender
            }),
            _react2['default'].createElement(
              'div',
              { className: 'variable-editor', style: variableStyle },
              _react2['default'].createElement(
                'div',
                {
                  className: 'variable-editor-title',
                  style: { cursor: variableOpen ? 'row-resize' : 'n-resize' },
                  onMouseDown: this._onVariableResizeStart
                },
                'Query Variables'
              ),
              _react2['default'].createElement(_VariableEditor.VariableEditor, {
                value: this.state.variables,
                variableToType: this.state.variableToType,
                onEdit: this._onEditVariables,
                onHintInformationRender: this._onHintInformationRender
              })
            )
          ),
          _react2['default'].createElement(
            'div',
            { className: 'resultWrap' },
            this.state.isWaitingForResponse && _react2['default'].createElement(
              'div',
              { className: 'spinner-container' },
              _react2['default'].createElement('div', { className: 'spinner' })
            ),
            _react2['default'].createElement(_ResultViewer.ResultViewer, { ref: 'result', value: this.state.response }),
            footer
          )
        )
      ),
      _react2['default'].createElement(
        'div',
        { className: 'docExplorerWrap', style: docWrapStyle },
        _react2['default'].createElement('div', {
          className: 'docExplorerResizer',
          onMouseDown: this._onDocsResizeStart
        }),
        _react2['default'].createElement(
          _DocExplorer.DocExplorer,
          { ref: 'docExplorer', schema: this.state.schema },
          _react2['default'].createElement(
            'div',
            { className: 'docExplorerHide', onClick: this._onToggleDocs },
            '✕'
          )
        )
      )
    );
  };

  // Private methods

  GraphiQL.prototype._ensureOfSchema = function _ensureOfSchema() {
    var _this2 = this;

    // Only perform introspection if a schema is not provided (undefined)
    if (this.state.schema !== undefined) {
      return;
    }

    var fetcher = this.props.fetcher;

    var fetch = fetcher({ query: _utilityIntrospectionQueries.introspectionQuery });
    if (!isPromise(fetch)) {
      console.error('Fetcher did not return a Promise for introspection.');
      return;
    }

    fetch.then(function (result) {
      if (result.data) {
        return result;
      }

      // Try the stock introspection query first, falling back on the
      // sans-subscriptions query for services which do not yet support it.
      var fetch2 = fetcher({ query: _utilityIntrospectionQueries.introspectionQuerySansSubscriptions });
      if (!isPromise(fetch)) {
        console.error('Fetcher did not return a Promise for introspection.');
        return;
      }
      return fetch2;
    }).then(function (result) {
      // If a schema was provided while this fetch was underway, then
      // satisfy the race condition by respecting the already
      // provided schema.
      if (_this2.state.schema !== undefined) {
        return;
      }

      if (result.data) {
        var schema = _graphql.buildClientSchema(result.data);
        var queryFacts = _utilityGetQueryFacts2['default'](_this2.state, schema, _this2.state.query);
        _this2.setState(_extends({ schema: schema }, queryFacts));
      } else {
        var responseString = typeof result === 'string' ? result : JSON.stringify(result, null, 2);
        _this2.setState({ response: responseString });
      }
    })['catch'](function (error) {
      _this2.setState({ response: error && (error.stack || String(error)) });
    });
  };

  GraphiQL.prototype._storageGet = function _storageGet(name) {
    return this._storage.getItem('graphiql:' + name);
  };

  GraphiQL.prototype._storageSet = function _storageSet(name, value) {
    this._storage.setItem('graphiql:' + name, value);
  };

  GraphiQL.prototype._fetchQuery = function _fetchQuery(query, variables, operationName, cb) {
    var _this3 = this;

    var fetcher = this.props.fetcher;
    var fetch = fetcher({ query: query, variables: variables, operationName: operationName });

    if (isPromise(fetch)) {
      // If fetcher returned a Promise, then call the callback when the promise
      // resolves, otherwise handle the error.
      fetch.then(cb)['catch'](function (error) {
        _this3.setState({
          isWaitingForResponse: false,
          response: error && (error.stack || String(error))
        });
      });
    } else if (isObservable(fetch)) {
      // If the fetcher returned an Observable, then subscribe to it, calling
      // the callback on each next value, and handling both errors and the
      // completion of the Observable. Returns a Subscription object.
      var subscription = fetch.subscribe({
        next: cb,
        error: function error(_error) {
          _this3.setState({
            isWaitingForResponse: false,
            response: _error && (_error.stack || String(_error)),
            subscription: null
          });
        },
        complete: function complete() {
          _this3.setState({
            isWaitingForResponse: false,
            subscription: null
          });
        }
      });

      return subscription;
    } else {
      this.setState({
        isWaitingForResponse: false,
        response: 'Fetcher did not return Promise or Observable.'
      });
    }
  };

  GraphiQL.prototype._runQueryAtCursor = function _runQueryAtCursor() {
    if (this.state.subscription) {
      this._stopQuery();
      return;
    }

    var operationName = undefined;
    var operations = this.state.operations;
    if (operations) {
      var editor = this.refs.queryEditor.getCodeMirror();
      var cursor = editor.getCursor();
      var cursorIndex = editor.indexFromPos(cursor);

      // Loop through all operations to see if one contains the cursor.
      for (var i = 0; i < operations.length; i++) {
        var operation = operations[i];
        if (operation.loc.start <= cursorIndex && operation.loc.end >= cursorIndex) {
          operationName = operation.name && operation.name.value;
          break;
        }
      }
    }

    this._runQuery(operationName);
  };

  GraphiQL.prototype._didClickDragBar = function _didClickDragBar(event) {
    // Only for primary unmodified clicks
    if (event.button !== 0 || event.ctrlKey) {
      return false;
    }
    var target = event.target;
    // We use codemirror's gutter as the drag bar.
    if (target.className.indexOf('CodeMirror-gutter') !== 0) {
      return false;
    }
    // Specifically the result window's drag bar.
    var resultWindow = _reactDom2['default'].findDOMNode(this.refs.result);
    while (target) {
      if (target === resultWindow) {
        return true;
      }
      target = target.parentNode;
    }
    return false;
  };

  return GraphiQL;
})(_react2['default'].Component);

exports.GraphiQL = GraphiQL;
GraphiQL.Logo = (function (_React$Component2) {
  _inherits(GraphiQLLogo, _React$Component2);

  function GraphiQLLogo() {
    _classCallCheck(this, GraphiQLLogo);

    _React$Component2.apply(this, arguments);
  }

  GraphiQLLogo.prototype.render = function render() {
    return _react2['default'].createElement(
      'div',
      { className: 'title' },
      this.props.children || _react2['default'].createElement(
        'span',
        null,
        'Graph',
        _react2['default'].createElement(
          'em',
          null,
          'i'
        ),
        'QL'
      )
    );
  };

  return GraphiQLLogo;
})(_react2['default'].Component);

// Configure the UI by providing this Component as a child of GraphiQL.
GraphiQL.Toolbar = (function (_React$Component3) {
  _inherits(GraphiQLToolbar, _React$Component3);

  function GraphiQLToolbar() {
    _classCallCheck(this, GraphiQLToolbar);

    _React$Component3.apply(this, arguments);
  }

  GraphiQLToolbar.prototype.render = function render() {
    return _react2['default'].createElement(
      'div',
      { className: 'toolbar' },
      this.props.children
    );
  };

  return GraphiQLToolbar;
})(_react2['default'].Component);

// Add a button to the Toolbar.
GraphiQL.ToolbarButton = _ToolbarButton.ToolbarButton;

// Configure the UI by providing this Component as a child of GraphiQL.
GraphiQL.Footer = (function (_React$Component4) {
  _inherits(GraphiQLFooter, _React$Component4);

  function GraphiQLFooter() {
    _classCallCheck(this, GraphiQLFooter);

    _React$Component4.apply(this, arguments);
  }

  GraphiQLFooter.prototype.render = function render() {
    return _react2['default'].createElement(
      'div',
      { className: 'footer' },
      this.props.children
    );
  };

  return GraphiQLFooter;
})(_react2['default'].Component);

var defaultQuery = '# Welcome to GraphiQL\n#\n# GraphiQL is an in-browser IDE for writing, validating, and\n# testing GraphQL queries.\n#\n# Type queries into this side of the screen, and you will\n# see intelligent typeaheads aware of the current GraphQL type schema and\n# live syntax and validation errors highlighted within the text.\n#\n# To bring up the auto-complete at any point, just press Ctrl-Space.\n#\n# Press the run button above, or Cmd-Enter to execute the query, and the result\n# will appear in the pane to the right.\n\n';

// Duck-type promise detection.
function isPromise(value) {
  return typeof value === 'object' && typeof value.then === 'function';
}

// Duck-type observable detection.
function isObservable(value) {
  return typeof value === 'object' && typeof value.subscribe === 'function';
}